@@ -416,12 +416,17 @@ cs_error_t confdb_dispatch (
 				break;
 
 			default:
-				coroipcc_dispatch_put (confdb_inst->handle);
-				error = CS_ERR_LIBRARY;
-				goto error_noput;
+				error = coroipcc_dispatch_put (confdb_inst->handle);
+				if (error == CS_OK) {
+					error = CS_ERR_LIBRARY;
+				}
+				goto error_put;
 				break;
 		}
-		coroipcc_dispatch_put (confdb_inst->handle);
+		error = coroipcc_dispatch_put (confdb_inst->handle);
+		if (error != CS_OK) {
+			goto error_put;
+		}
 
 		/*
 		 * Determine if more messages should be processed
@@ -433,7 +438,6 @@ cs_error_t confdb_dispatch (
 
 error_put:
 	(void)hdb_handle_put (&confdb_handle_t_db, handle);
-error_noput:
 	return (error);
 }
 
